﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Protocols;
using ProtoBuf;
using System.IO;
using System.Data;
using System.Collections;
using Utility;

namespace GameLib {
    enum EChangeType {
        Null = 0,
        Day = 1,    //每天计数
        Week = 2,   //每周计数
        Month = 3,  //每月计数
    };

    public enum ECounterType {
        Null = 0,
        SignIn = 1,     //签到
        DailyTask = 2,  //日常任务

    }

    /// <summary>
    /// 计数器条目
    /// </summary>
    [ProtoContract]
    class stCounterItem {
        //数量
        [ProtoMember(1)] public int count = 0;
        //上次变化时间
        [ProtoMember(2)] public DateTime lastTime = DateTime.MinValue;
        //计数类型
        [ProtoMember(3)] public ECounterType counterType = ECounterType.Null;
        //变化类型
        [ProtoMember(4)] public EChangeType changeType = EChangeType.Null;
    };

    [ProtoContract]
    public class Counter {
        public Counter() { }

        //存储数据=============================================== 

        /// <summary>
        /// 计数器条目
        /// </summary>
        [ProtoMember(1)]
        Dictionary<ECounterType, stCounterItem> mapCounters = new Dictionary<ECounterType, stCounterItem>();

        //=======================================================

        public int GetCounter(ECounterType id) {
            if (!mapCounters.ContainsKey(id)) {
                return 0;
            }

            DateTime currentDateTime = DateTime.Now;

            var item = mapCounters[id];

            if (item.changeType == EChangeType.Day) {
                if (!item.lastTime.IsSameDay(currentDateTime)) {
                    item.count = 0;
                    item.lastTime = currentDateTime;
                }
            }

            if (item.changeType == EChangeType.Week) {
                if (!item.lastTime.IsSameWeek(currentDateTime)) {
                    item.count = 0;
                    item.lastTime = currentDateTime;
                }
            }

            if (item.changeType == EChangeType.Month) {
                if (!item.lastTime.IsSameMonth(currentDateTime)) {
                    item.count = 0;
                    item.lastTime = currentDateTime;
                }
            }

            return item.count;
        }

        /// <summary>
        /// 需要按天切换的使用该函数进行计数
        /// </summary>
        /// <param name="id"></param>
        /// <param name="num"></param>
        /// <returns></returns>
        public bool AddCounterDay(ECounterType id, int num = 1) {
            if (mapCounters.ContainsKey(id)) {
                int count = GetCounter(id) + num;
                var item = mapCounters[id];
                item.count = count;
            }
            else {
                stCounterItem item = new stCounterItem();
                item.counterType = id;
                item.changeType = EChangeType.Day;
                item.count = num;
                item.lastTime = DateTime.Now;
                mapCounters.Add(id, item);
            }

            return true;
        }

        /// <summary>
        /// 需要按周切换的使用该函数进行计数
        /// </summary>
        /// <param name="id"></param>
        /// <param name="num"></param>
        /// <returns></returns>
        public bool AddCounterWeek(ECounterType id, int num = 1) {
            if (mapCounters.ContainsKey(id)) {
                int count = GetCounter(id) + num;
                var item = mapCounters[id];
                item.count = count;
            }
            else {
                stCounterItem item = new stCounterItem();
                item.counterType = id;
                item.changeType = EChangeType.Week;
                item.count = num;
                item.lastTime = DateTime.Now;
                mapCounters.Add(id, item);
            }

            return true;
        }

        /// <summary>
        /// 需要按月切换的使用该函数进行计数
        /// </summary>
        /// <param name="id"></param>
        /// <param name="num"></param>
        /// <returns></returns>
        public bool AddCounterMonth(ECounterType id, int num = 1) {
            if (mapCounters.ContainsKey(id)) {
                int count = GetCounter(id) + num;
                var item = mapCounters[id];
                item.count = count;
            }
            else {
                stCounterItem item = new stCounterItem();
                item.counterType = id;
                item.changeType = EChangeType.Month;
                item.count = num;
                item.lastTime = DateTime.Now;
                mapCounters.Add(id, item);
            }

            return true;
        }
    }
}
